package db

import (
	"context"
	"fmt"
	"github.com/caixw/lib.go/conv"
	"github.com/sirupsen/logrus"
	"gorm.io/gorm"
	"reflect"
	"strconv"
	"time"
	"youngee_m_api/consts"
	"youngee_m_api/model/common_model"
	"youngee_m_api/model/gorm_model"
	"youngee_m_api/model/http_model"
	"youngee_m_api/pack"
	"youngee_m_api/util"
)

func GetFullProjectList(ctx context.Context, pageSize, pageNum int32, condition *common_model.ProjectCondition) ([]*gorm_model.ProjectInfo, int64, error) {
	db := GetReadDB(ctx)
	db = db.Debug().Model(gorm_model.ProjectInfo{})
	// 根据Project条件过滤
	conditionType := reflect.TypeOf(condition).Elem()
	conditionValue := reflect.ValueOf(condition).Elem()
	for i := 0; i < conditionType.NumField(); i++ {
		field := conditionType.Field(i)
		tag := field.Tag.Get("condition")
		value := conditionValue.FieldByName(field.Name)
		if tag == "project_status" && util.IsBlank(value) {
			db = db.Where(fmt.Sprintf("project_status != 3"))
			db = db.Where(fmt.Sprintf("project_status != 5"))
			db = db.Where(fmt.Sprintf("project_status != 7"))
		}
		if !util.IsBlank(value) && tag != "updated_at" && tag != "project_name" {
			db = db.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
		}
		if tag == "updated_at" && value.Interface() != "0" {
			db = db.Where(fmt.Sprintf("updated_at like '%s%%'", value.Interface()))
		}
		if tag == "project_name" && !util.IsBlank(value) {
			//db = db.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
			db = db.Where(fmt.Sprintf("project_name like '%%%s%%'", value.Interface()))
		}
	}
	// 查询总数
	var total int64
	var fullProjects []*gorm_model.ProjectInfo
	if err := db.Count(&total).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetFullProjectList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	// 查询该页数据
	limit := pageSize
	offset := pageSize * pageNum // assert pageNum start with 0
	err := db.Order("updated_at").Limit(int(limit)).Offset(int(offset)).Find(&fullProjects).Error
	if err != nil {
		logrus.WithContext(ctx).Errorf("[GetFullProjectList] error query mysql find, err:%+v", err)
		return nil, 0, err
	}
	return fullProjects, total, nil
}

func GetProjectDetail(ctx context.Context, projectID int64) (*gorm_model.ProjectInfo, error) {
	db := GetReadDB(ctx)
	var ProjectDetail *gorm_model.ProjectInfo
	err := db.Where("project_id = ?", projectID).First(&ProjectDetail).Error
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		} else {
			return nil, err
		}
	}
	return ProjectDetail, nil
}

func GetRecruitStrategys(ctx context.Context, ProjectID int64) ([]gorm_model.RecruitStrategy, error) {
	db := GetReadDB(ctx)
	RecruitStrategys := []gorm_model.RecruitStrategy{}
	err := db.Where("project_id=?", ProjectID).Find(&RecruitStrategys).Error
	if err != nil {
		return nil, err
	}
	return RecruitStrategys, nil
}

func GetProjectPhoto(ctx context.Context, ProjectID int64) ([]gorm_model.ProjectPhoto, error) {
	db := GetReadDB(ctx)
	var ProjectPhoto []gorm_model.ProjectPhoto
	err := db.Where("project_id=?", ProjectID).Find(&ProjectPhoto).Error
	if err != nil {
		return nil, err
	}
	return ProjectPhoto, nil
}

func UpdateProject(ctx context.Context, project gorm_model.ProjectInfo) (*int64, error) {
	db := GetReadDB(ctx)
	err := db.Model(&project).Updates(project).Error
	if err != nil {
		return nil, err
	}
	return &project.ProjectID, nil
}

func ApproveProject(ctx context.Context, projectId string, isApprove int64) (error error, message string) {
	project_id := conv.MustInt64(projectId, 0)
	db := GetReadDB(ctx)
	projectInfo := gorm_model.ProjectInfo{}
	db = db.Debug().Model(gorm_model.ProjectInfo{}).Where("project_id = ?", project_id).First(&projectInfo)
	t := time.Now()
	if isApprove == 1 && projectInfo.ProjectStatus == 2 {
		err := db.Updates(gorm_model.ProjectInfo{
			ProjectStatus: 4,
			FinishAt:      &t,
		}).Error
		message = "审核通过"
		if err != nil {
			logrus.Println("DB AutoUpdateStatus error :", err)
			return err, ""
		}
	} else if isApprove == 2 && projectInfo.ProjectStatus == 2 {
		err := db.Update("project_status", 8).Error
		message = "项目存在风险已禁止发布"
		if err != nil {
			logrus.Println("DB AutoUpdateStatus error :", err)
			return err, ""
		}
	} else {
		return nil, "操作失败"
	}
	return nil, message
}

func GetAllProject(ctx context.Context, pageSize, pageNum int32) ([]*http_model.GetAllProjectPreview, int64, error) {
	db := GetReadDB(ctx)
	db = db.Debug().Model(gorm_model.ProjectInfo{}).Where("project_status = 1 or project_status = 6")
	var allProjects []*http_model.GetAllProjectPreview
	// 查询总数
	var total int64
	var fullProjects []*gorm_model.ProjectInfo
	if err := db.Count(&total).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetAllProject] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	// 查询该页数据
	limit := pageSize
	offset := pageSize * pageNum // assert pageNum start with 0
	err := db.Order("updated_at desc").Limit(int(limit)).Offset(int(offset)).Find(&fullProjects).Error
	if err != nil {
		logrus.WithContext(ctx).Errorf("[GetAllProject] error query mysql find, err:%+v", err)
		return nil, 0, err
	}
	var EnterpriseIDs []int64
	for _, fullProject := range fullProjects {
		EnterpriseIDs = append(EnterpriseIDs, fullProject.EnterpriseID)
	}
	EnterpriseIDs = util.RemoveRepByMap(EnterpriseIDs)
	userMemo := map[int64]string{}
	phoneMemo := map[int64]string{}
	for _, EnterpriseID := range EnterpriseIDs {
		userMemo[EnterpriseID] = GetUsernameEnterpriseID(ctx, EnterpriseID)
	}
	for _, EnterpriseID := range EnterpriseIDs {
		phoneMemo[EnterpriseID] = GetPhoneByEnterpriseID(ctx, EnterpriseID)
	}
	for _, fullProject := range fullProjects {
		allProject := new(http_model.GetAllProjectPreview)
		allProject.ProjectUpdated = conv.MustString(fullProject.UpdatedAt, "")
		allProject.ProjectStatus = conv.MustString(fullProject.ProjectStatus, "")
		allProject.ProjectId = conv.MustString(fullProject.ProjectID, "")
		allProject.EnterpriseID = fullProject.EnterpriseID
		allProject.Username = userMemo[fullProject.EnterpriseID]
		allProject.Phone = phoneMemo[fullProject.EnterpriseID]
		allProjects = append(allProjects, allProject)
	}
	return allProjects, total, nil
}

// GetPhoneByEnterpriseID 根据企业ID查找用户联系方式
func GetPhoneByEnterpriseID(ctx context.Context, enterpriseID int64) string {
	db := GetReadDB(ctx)
	var Enterprise gorm_model.Enterprise
	// 通过企业ID获得用户ID
	db = db.Model([]gorm_model.Enterprise{}).Where("enterprise_id", enterpriseID).First(&Enterprise)
	userId := Enterprise.UserID
	db1 := GetReadDB(ctx)
	var user gorm_model.YounggeeUser
	db1 = db1.Model([]gorm_model.YounggeeUser{}).Where("id", userId).First(&user)
	phone := user.Phone
	return phone
}

// GetUsernameEnterpriseID 根据企业ID查找用户名称
func GetUsernameEnterpriseID(ctx context.Context, enterpriseID int64) string {
	db := GetReadDB(ctx)
	var Enterprise gorm_model.Enterprise
	// 通过企业ID获得用户ID
	db = db.Model([]gorm_model.Enterprise{}).Where("enterprise_id", enterpriseID).First(&Enterprise)
	userId := Enterprise.UserID
	db1 := GetReadDB(ctx)
	var user gorm_model.YounggeeUser
	db1 = db1.Model([]gorm_model.YounggeeUser{}).Where("id", userId).First(&user)
	username := user.Username
	return username
}

func GetProjectTaskList(ctx context.Context, projectID string, pageSize, pageNum int64, conditions *common_model.TaskConditions) ([]*http_model.ProjectTaskInfo, int64, error) {
	db := GetReadDB(ctx)
	// 查询task表信息
	db = db.Debug().Model(gorm_model.YoungeeTaskInfo{})
	// 根据Project条件过滤
	conditionType := reflect.TypeOf(conditions).Elem()
	conditionValue := reflect.ValueOf(conditions).Elem()
	for i := 0; i < conditionType.NumField(); i++ {
		field := conditionType.Field(i)
		tag := field.Tag.Get("condition")
		value := conditionValue.FieldByName(field.Name)
		if !util.IsBlank(value) && tag != "platform_nickname" {
			db = db.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
		} else if tag == "platform_nickname" {
			continue
		}
	}
	var taskInfos []gorm_model.YoungeeTaskInfo
	db = db.Model(gorm_model.YoungeeTaskInfo{})
	// 查询总数
	var totalTask int64
	if err := db.Count(&totalTask).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetProjectTaskList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	db.Order("task_id").Find(&taskInfos)

	// 查询账号id
	var accountIds []int
	taskMap := make(map[int]gorm_model.YoungeeTaskInfo)
	for _, taskInfo := range taskInfos {
		accountIds = append(accountIds, taskInfo.AccountId)
		taskMap[taskInfo.AccountId] = taskInfo
	}
	db1 := GetReadDB(ctx)
	db1 = db1.Debug().Model(gorm_model.YoungeePlatformAccountInfo{})
	// 根据Project条件过滤
	conditionType2 := reflect.TypeOf(conditions).Elem()
	conditionValue2 := reflect.ValueOf(conditions).Elem()
	for i := 0; i < conditionType2.NumField(); i++ {
		field := conditionType2.Field(i)
		tag := field.Tag.Get("condition")
		value := conditionValue2.FieldByName(field.Name)
		if !util.IsBlank(value) && tag == "platform_nickname" { // input:1		database:1,2,3    string
			db1 = db1.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
		}
	}
	var accountInfos []gorm_model.YoungeePlatformAccountInfo
	db1 = db1.Model(gorm_model.YoungeePlatformAccountInfo{}).Where("account_id IN ?", accountIds).Find(&accountInfos)
	// 查询总数
	var totalAccount int64
	if err := db1.Count(&totalAccount).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetProjectTaskList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	var misNum int64
	if totalAccount > totalTask {
		misNum = totalAccount - totalTask
	} else {
		misNum = totalTask - totalAccount
	}
	logrus.Println("totalAccount,totalTask,misNum:", totalAccount, totalTask, misNum)
	// 查询该页数据
	limit := pageSize + misNum
	offset := pageSize * pageNum // assert pageNum start with 0
	err := db.Order("task_id").Limit(int(limit)).Offset(int(offset)).Error
	if err != nil {
		logrus.WithContext(ctx).Errorf("[GetProjectTaskList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}

	accountMap := make(map[int]gorm_model.YoungeePlatformAccountInfo)
	for _, accountInfo := range accountInfos {
		accountMap[accountInfo.AccountID] = accountInfo
	}
	var taskAccounts []*http_model.TaskAccount
	var projectTasks []*http_model.ProjectTaskInfo
	for _, accountId := range accountIds {
		taskAccount := new(http_model.TaskAccount)
		_, ok := taskMap[accountId]
		_, ok2 := accountMap[accountId]
		if ok && ok2 {
			taskAccount.Task = taskMap[accountId]
			taskAccount.Account = accountMap[accountId]
		}
		taskAccounts = append(taskAccounts, taskAccount)
	}

	projectTasks = pack.TaskAccountToTaskInfo(taskAccounts)
	var fullProjectTasks []*http_model.ProjectTaskInfo
	// 删除只存在于一个表中的元素
	for i := 0; i < len(projectTasks); i++ {
		if projectTasks[i].TaskID != 0 {
			fullProjectTasks = append(fullProjectTasks, projectTasks[i])
		}
	}
	var total int64
	if totalTask > totalAccount {
		total = totalAccount
	} else {
		total = totalTask
	}
	return fullProjectTasks, total, nil
}

func CreateProject(ctx context.Context, projectInfo gorm_model.ProjectInfo) (*int64, error) {
	db := GetWriteDB(ctx)
	err := db.Create(&projectInfo).Error
	if err != nil {
		return nil, err
	}
	return &projectInfo.ProjectID, nil
}

func ProjectHandle(ctx context.Context, data http_model.ProjectHandleRequest) error {
	err := ChangeProjectStatus(ctx, data.ProjectID, data.ProjectStatus)
	if err != nil {
		logrus.WithContext(ctx).Errorf("[project] call ChangeProjectStatus error,err:%+v", err)
		return err
	}
	return nil
}

func ChangeProjectStatus(ctx context.Context, projectID int64, projectStatus string) error {
	db := GetReadDB(ctx)
	projectInfo := gorm_model.ProjectInfo{}
	var project_status int64
	if projectStatus == "待支付" {
		project_status = 7
	} else if projectStatus == "创建中" {
		project_status = 2
	}
	if err := db.Debug().Model(&projectInfo).
		Where("project_id = ?", projectID).
		Updates(gorm_model.ProjectInfo{ProjectStatus: project_status}).
		Error; err != nil {
		logrus.WithContext(ctx).Errorf("[ChangeProjectStatus] error query mysql total, err:%+v", err)
		return err
	}
	return nil
}

func GetProjectRecords(ctx context.Context, userId int64, pageSize, pageNum int32) (*http_model.ProjectRecordsData, error) {
	db := GetReadDB(ctx)
	enterpriseID := GetEnterpriseIDByUserId(ctx, userId)
	var projectInfos []*gorm_model.ProjectInfo
	db = db.Model(&gorm_model.ProjectInfo{}).Where("enterprise_id = ?", enterpriseID)
	// 查询总数
	var total int64
	if err := db.Count(&total).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetProjectRecords] error query mysql total, err:%+v", err)
		return nil, err
	}
	// 查询该页数据
	limit := pageSize
	offset := pageSize * pageNum // assert pageNum start with 0
	err := db.Order("updated_at desc").Limit(int(limit)).Offset(int(offset)).Find(&projectInfos).Error
	if err != nil {
		logrus.WithContext(ctx).Errorf("[GetProjectRecords] error query mysql limit, err:%+v", err)
		return nil, err
	}
	var projectRecords []*http_model.ProjectRecordsPreview
	for _, projectInfo := range projectInfos {
		projectRecordsPreview := new(http_model.ProjectRecordsPreview)
		projectRecordsPreview.ProjectID = projectInfo.ProjectID
		projectRecordsPreview.ProjectName = projectInfo.ProjectName
		projectRecordsPreview.ProjectType = consts.GetProjectType(projectInfo.ProjectType)
		projectRecordsPreview.UpdatedAt = conv.MustString(projectInfo.UpdatedAt, "")[:19]
		projectRecordsPreview.ProjectStatus = consts.GetProjectStatus(projectInfo.ProjectStatus)
		projectRecords = append(projectRecords, projectRecordsPreview)
	}
	projectRecordsData := http_model.ProjectRecordsData{}
	projectRecordsData.ProjectRecordsPreview = projectRecords
	projectRecordsData.Total = strconv.FormatInt(total, 10)
	return &projectRecordsData, nil
}
